@@ -122,7 +122,7 @@ seed_gi_importer_handle_enum (JSContextRef ctx,
   JSObjectRef enum_class;
   guint num_vals, i, j;
   gsize name_len;
-  glong value;
+  gint value; // TODO: investigate what's up with the glong/gint mystery here
   gchar *name;
   GIValueInfo *val;
 
@@ -348,7 +348,7 @@ seed_gi_importer_handle_constant (JSContextRef ctx,
 
   g_constant_info_get_value (info, &argument);
   constant_value =
-    seed_gi_argument_make_js (ctx, &argument, constant_type, exception);
+    seed_value_from_gi_argument (ctx, &argument, constant_type, exception);
   seed_object_set_property (ctx, namespace_ref,
 			    g_base_info_get_name ((GIBaseInfo *) info),
 			    constant_value);
@@ -371,7 +371,7 @@ seed_gi_importer_get_version (JSContextRef ctx,
   return version;
 }
 
-static JSObjectRef
+JSObjectRef
 seed_gi_importer_do_namespace (JSContextRef ctx,
 			       gchar * namespace, JSValueRef * exception)
 {
@@ -379,10 +379,13 @@ seed_gi_importer_do_namespace (JSContextRef ctx,
   JSObjectRef namespace_ref;
   GError *e = NULL;
   guint n, i;
-  gchar *version;
+  gchar *version = NULL;
   gchar *jsextension;
   JSStringRef extension_script;
 
+  if (gi_imports == NULL)
+    gi_imports = g_hash_table_new (g_str_hash, g_str_equal);
+
   if ((namespace_ref = g_hash_table_lookup (gi_imports, namespace)))
     {
       SEED_NOTE (IMPORTER, "Using existing namespace ref (%p) for %s",
@@ -390,7 +393,8 @@ seed_gi_importer_do_namespace (JSContextRef ctx,
       return namespace_ref;
     }
 
-  version = seed_gi_importer_get_version (ctx, namespace, exception);
+  if (gi_importer_versions != NULL)
+    version = seed_gi_importer_get_version (ctx, namespace, exception);
   if (!g_irepository_require (NULL, namespace, version, 0, &e))
     {
       seed_make_exception_from_gerror (ctx, exception, e);
@@ -398,7 +402,8 @@ seed_gi_importer_do_namespace (JSContextRef ctx,
       g_free (version);
       return NULL;
     }
-  g_free (version);
+  if (version != NULL)
+    g_free (version);
 
   n = g_irepository_get_n_infos (NULL, namespace);
 
@@ -573,21 +578,31 @@ seed_importer_get_search_path (JSContextRef ctx, JSValueRef * exception)
   return path;
 }
 
+/* forward reference */
+static JSObjectRef
+seed_importer_handle_file (JSContextRef ctx,
+			   const gchar * dir,
+			   const gchar * file, 
+			   JSValueRef * exception);
+
+
+
 static JSObjectRef
 seed_importer_handle_native_module (JSContextRef ctx,
 				    const gchar * dir,
-				    const gchar * file,
+				    const gchar * prop,
 				    JSValueRef * exception)
 {
   GModule *module;
   JSObjectRef module_obj;
   SeedModuleInitCallback init;
-  gchar *file_path = g_strconcat (dir, "/", file, NULL);
+  gchar *file_path = g_strconcat (dir, "/libseed_", prop, ".", G_MODULE_SUFFIX, NULL);
 
   SEED_NOTE (IMPORTER, "Trying native module: %s", file_path);
 
   if ((module_obj = g_hash_table_lookup (file_imports, file_path)))
     {
+      SEED_NOTE (IMPORTER, "Using existing global");
       g_free (file_path);
       return module_obj;
     }
@@ -609,6 +624,11 @@ seed_importer_handle_native_module (JSContextRef ctx,
   g_hash_table_insert (file_imports, file_path, module_obj);
   SEED_NOTE (IMPORTER, "Loaded native module");
 
+  //protect module_obj since the GC won't find the module in our file_imports cache
+  JSValueProtect (ctx, module_obj);
+  
+  file_path = g_strconcat ("libseed_", prop, ".js", NULL);
+  seed_importer_handle_file (ctx, dir, file_path, exception);
   g_free (file_path);
 
   return module_obj;
@@ -674,7 +694,7 @@ seed_importer_handle_file (JSContextRef ctx,
   g_free (contents);
 
   file_contents = JSStringCreateWithUTF8CString (walk);
-  file_name = JSStringCreateWithUTF8CString (file);
+  file_name = JSStringCreateWithUTF8CString (canonical);
 
   nctx = JSGlobalContextCreateInGroup (context_group, 0);
   seed_prepare_global_context (nctx);
@@ -718,79 +738,79 @@ seed_importer_handle_file (JSContextRef ctx,
 }
 
 static JSObjectRef
-seed_importer_search (JSContextRef ctx, gchar * prop, JSValueRef * exception)
+seed_importer_search_dirs (JSContextRef ctx, GSList *path, gchar *prop, JSValueRef *exception)
 {
+    GSList *walk;
+    JSObjectRef ret, global;
+    JSValueRef script_path_prop;
+    gchar *prop_as_lib, *prop_as_js, *script_path;
 
-  GDir *dir;
-  GError *e;
-  gchar *mentry;
-  GSList *path, *walk;
-  JSObjectRef ret;
-  gchar *prop_as_lib =
-    g_strconcat ("libseed_", prop, ".", G_MODULE_SUFFIX, NULL);
-  gsize i, mentrylen;
+    prop_as_lib = g_strconcat ("libseed_", prop, ".", G_MODULE_SUFFIX, NULL);
+    prop_as_js = g_strconcat (prop, ".js", NULL);
 
-  path = seed_importer_get_search_path (ctx, exception);
+    // get the current script_path      
+    global = JSContextGetGlobalObject (ctx);
+    script_path_prop = seed_object_get_property (ctx, global, "__script_path__");
+    if (script_path_prop==NULL || JSValueIsUndefined (ctx, script_path_prop))
+        script_path = NULL;
+    else
+        script_path = seed_value_to_string (ctx, script_path_prop, exception);
 
-  walk = path;
-  while (walk)
-    {
-      e = NULL;
-      const gchar *entry;
+    ret = NULL;
+    walk = path;
+    while (walk) {
+        gchar *test_path = walk->data;
+        gchar *file_path;
+        
+        // replace '.' with current script_path if not null
+        if(script_path && !g_strcmp0(".",test_path))
+            test_path = script_path;
 
-      dir = g_dir_open ((gchar *) walk->data, 0, &e);
-      if (e)
-	{
-	  g_error_free (e);
+        // check if prop is a file or dir (imports['foo.js'] or imports.mydir)
+        file_path = g_build_filename (test_path, prop, NULL);
+        if (g_file_test (file_path, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_DIR)) {
+            ret = seed_importer_handle_file (ctx, test_path, prop, exception);
+            g_free (file_path);
+            break;
+        }
+        g_free (file_path);
 
-	  walk = walk->next;
-	  continue;
-	}
-      while ((entry = g_dir_read_name (dir)))
-	{
-	  mentry = g_strdup (entry);
+        // check if prop is file ending with '.js'
+        file_path = g_build_filename (test_path, prop_as_js, NULL);
+        if (g_file_test (file_path, G_FILE_TEST_IS_REGULAR)) {
+            ret = seed_importer_handle_file (ctx, test_path, prop_as_js, exception);
+            g_free (file_path);
+            break;
+        }
+        g_free (file_path);
+      
+        // check if file is native module
+        file_path = g_build_filename (test_path, prop_as_lib, NULL);
+        if (g_file_test (file_path, G_FILE_TEST_IS_REGULAR)) {
+            ret = seed_importer_handle_native_module (ctx, test_path, prop, exception);
+            g_free (file_path);
+            break;
+        }
+        g_free (file_path);
 
-	  mentrylen = strlen (mentry);
-	  for (i = 0; i < mentrylen; i++)
-	    {
-	      if (mentry[i] == '.')
-		mentry[i] = '\0';
-	    }
-	  if (!g_strcmp0 (mentry, prop))
-	    {
-	      ret =
-		seed_importer_handle_file (ctx, walk->data, entry, exception);
-
-	      g_dir_close (dir);
-	      g_free (mentry);
-	      g_free (prop_as_lib);
-	      seed_importer_free_search_path (path);
-
-	      return ret;
-	    }
-	  else if (!g_strcmp0 (entry, prop_as_lib))
-	    {
-	      ret =
-		seed_importer_handle_native_module (ctx, walk->data, entry,
-						    exception);
-	      g_dir_close (dir);
-	      g_free (mentry);
-	      g_free (prop_as_lib);
-	      seed_importer_free_search_path (path);
-
-	      return ret;
-	    }
-
-	  g_free (mentry);
-	}
-      g_dir_close (dir);
-
-      walk = walk->next;
+        walk = walk->next;
     }
 
-  seed_importer_free_search_path (path);
-  g_free (prop_as_lib);
-  return NULL;
+    g_free (prop_as_lib);
+    g_free (prop_as_js);
+    g_free (script_path);
+    
+    return ret;
+}
+
+static JSObjectRef
+seed_importer_search (JSContextRef ctx, gchar *prop, JSValueRef *exception)
+{
+    JSObjectRef ret = NULL;
+    GSList *path = seed_importer_get_search_path (ctx, exception);
+    ret = seed_importer_search_dirs (ctx, path, prop, exception);
+    seed_importer_free_search_path (path);
+    return ret;
 }
 
 static JSValueRef
@@ -820,58 +840,30 @@ seed_importer_get_property (JSContextRef ctx,
 
 static JSValueRef
 seed_importer_dir_get_property (JSContextRef ctx,
-				JSObjectRef object,
-				JSStringRef property_name,
-				JSValueRef * exception)
+                               JSObjectRef object,
+                               JSStringRef property_name,
+                               JSValueRef * exception)
 {
-  GError *e = NULL;
-  GDir *dir;
-  guint len, i;
-  gsize mentrylen;
-  const gchar *entry;
-  gchar *dir_path, *prop, *mentry;
-  JSObjectRef ret;
+    guint len;
+    gchar *prop;
+    GSList path;
 
-  dir_path = JSObjectGetPrivate (object);
+    path.data = JSObjectGetPrivate (object);
+    path.next = NULL;
 
-  len = JSStringGetMaximumUTF8CStringSize (property_name);
-  prop = g_alloca (len * sizeof (gchar));
-  JSStringGetUTF8CString (property_name, prop, len);
+    len = JSStringGetMaximumUTF8CStringSize (property_name);
+    prop = g_alloca (len * sizeof (gchar));
+    JSStringGetUTF8CString (property_name, prop, len);
 
-  // TODO: GError
-  dir = g_dir_open (dir_path, 0, &e);
-  if (e)
-    {
-      seed_make_exception_from_gerror (ctx, exception, e);
-      g_error_free (e);
+    /* These prevent print(imports.somefile) running "somefile/toString.js" 
+       Which is more than a little unexpected.. */
 
-      return NULL;
-    }
-  while ((entry = g_dir_read_name (dir)))
-    {
-      mentry = g_strdup (entry);
-      mentrylen = strlen (mentry);
-
-      for (i = 0; i < mentrylen; i++)
-	{
-	  if (mentry[i] == '.')
-	    mentry[i] = '\0';
-	}
-
-      if (!g_strcmp0 (mentry, prop))
-	{
-
-	  ret = seed_importer_handle_file (ctx, dir_path, entry, exception);
-	  g_dir_close (dir);
-	  g_free (mentry);
-
-	  return ret;
-	}
-      g_free (mentry);
-    }
-  g_dir_close (dir);
-
-  return NULL;
+    if (!g_strcmp0 (prop, "toString"))
+        return NULL;
+    if (!g_strcmp0 (prop, "valueOf"))
+        return NULL;
+    
+    return seed_importer_search_dirs(ctx, &path, prop, exception);
 }
 
 static void
