@@ -171,7 +171,7 @@ seed_field_get_value (JSContextRef ctx,
     }
 
   // Maybe need to release argument.
-  ret = seed_gi_argument_make_js (ctx, &field_value, field_type, exception);
+  ret = seed_value_from_gi_argument (ctx, &field_value, field_type, exception);
   if (field_type)
     g_base_info_unref ((GIBaseInfo *) field_type);
   return ret;
@@ -241,7 +241,7 @@ seed_union_set_property (JSContextRef context,
 
   field_type = g_field_info_get_type (field);
 
-  seed_gi_make_argument (context, value, field_type, &field_value, exception);
+  seed_value_to_gi_argument (context, value, field_type, &field_value, exception);
   ret = g_field_info_set_field (field, priv->pointer, &field_value);
 
   g_base_info_unref ((GIBaseInfo *) field_type);
@@ -283,7 +283,7 @@ seed_struct_set_property (JSContextRef context,
 
   field_type = g_field_info_get_type (field);
 
-  seed_gi_make_argument (context, value, field_type, &field_value, exception);
+  seed_value_to_gi_argument (context, value, field_type, &field_value, exception);
   ret = g_field_info_set_field (field, priv->pointer, &field_value);
 
   if (!ret) 
@@ -721,7 +721,7 @@ seed_construct_struct_type_with_parameters (JSContextRef ctx,
               return (JSObjectRef) JSValueMakeNull (ctx);
             }
           SEED_NOTE (CONSTRUCTION, "Created a GValue  struct");
-          seed_gvalue_from_seed_value (ctx,   parameters , 0,  gval, exception);
+          seed_value_to_gvalue (ctx,   parameters , 0,  gval, exception);
           ret = seed_make_struct (ctx, (gpointer)gval, info);
           return ret;
         }
@@ -785,8 +785,8 @@ seed_construct_struct_type_with_parameters (JSContextRef ctx,
 					  (JSObjectRef) parameters,
 					  jsprop_name, NULL);
 
-      seed_gi_make_argument (ctx, jsprop_value, field_type, &field_value,
-			     exception);
+      seed_value_to_gi_argument (ctx, jsprop_value, field_type, &field_value,
+				 exception);
       set_ret = g_field_info_set_field (field, object, &field_value);
 
 	  if (!set_ret) 
